import {Component, ElementRef, Input, ViewChild} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {Meta, Title} from '@angular/platform-browser';
import {OrganizationService} from '../../services/organization.service';
import {OrganizationInfo} from '../../utils/entities/organizationInfo';
import {ReportsService} from '../../services/reports.service';
import {FetchProjects} from '../../utils/fetchEntitiesClasses/fetchProjects.class';
import {SearchResearchResultsService} from '../../services/searchResearchResults.service';
import {SearchDataprovidersService} from '../../services/searchDataproviders.service';
import {SearchProjectsService} from '../../services/searchProjects.service';
import {ErrorCodes} from '../../utils/properties/errorCodes';
import {ProjectsInModalComponent} from '../landing-utils/projects-in-modal.component';
import {RouterHelper} from '../../utils/routerHelper.class';

import {ModalLoading} from '../../utils/modal/loading.component';
import {PiwikService} from '../../utils/piwik/piwik.service';
import {StringUtils} from '../../utils/string-utils.class';
import {EnvProperties} from '../../utils/properties/env-properties';
import {SEOService} from '../../sharedComponents/SEO/SEO.service';
import {HelperFunctions} from "../../utils/HelperFunctions.class";
import {HelperService} from "../../utils/helper/helper.service";
import {Location} from "@angular/common";

interface Total {
  publications: number;
  datasets: number;
  software: number;
  other: number;
  results: number;
  dataproviders: number;
}

@Component({
  selector: 'organization',
  templateUrl: 'organization.component.html',
})
export class OrganizationComponent {
  @Input() piwikSiteId = null;
  @Input() communityId = null;
  
  public organizationInfo: OrganizationInfo;
  public organizationId: string;
  
  // Message variables
  public warningMessage = "";
  public errorMessage = "";
  public showLoading: boolean = true;
  
  // CSV variables
  public downloadURLAPI: string;
  public csvProjectParamsHead: string;
  public csvPublicationParamsHead: string;
  public csvParamsTail: string;
  
  // Active tab variable for responsiveness
  public activeTab: string = "Publications";
  
  // Variables for publications, research data, projects, dataproviders tabs
  public total: Total = {
    publications: 0,
    datasets: 0,
    software: 0,
    other: 0,
    results: 0,
    dataproviders: 0
  };
  public fetchProjects: FetchProjects;
  // Variables for projects query (query results only if projects tab is clicked)
  @ViewChild(ProjectsInModalComponent) projectsInModalComponent: ProjectsInModalComponent;
  
  @ViewChild(ModalLoading) loading: ModalLoading;
  // Alert box when CSV: Project Publications for a funder is requested
  @ViewChild('AlertModalApplyAll') alertApplyAll;
  // Alert box when something is wrong with CSV requests
  @ViewChild('AlertModalCsvError') alertCsvError;
  
  public routerHelper: RouterHelper = new RouterHelper();
  public errorCodes: ErrorCodes = new ErrorCodes();
  public pageContents = null;
  public divContents = null;
  
  // Helper variables to specify funder in downloadPublicationsFile function
  public contentTypes: [string,string][] =[
    ['results', 'all research outcomes'],
    ['publications', 'publications'],
    ['datasets', 'research data'],
    ['software', 'software'],
    ['other', 'other research products'],
  ];
  public funderContentType: string = '';
  public funder: any = "";
  private funderId: string;
  private funderCount: number;
  sub: any;
  infoSub: any;
  piwiksub: any;
  downloadFileSub: any;
  downloadFilePiwikSub: any;
  countProjectsSub: any;
  countPublSub: any;
  downloadProjectPublSub: any;
  properties: EnvProperties;
  public indexUpdateDate: Date;
  public showFeedback: boolean = false;
  public feedbackFields: string [] = ['Name', 'Country', 'Other'];
  
  @ViewChild('AlertModalDeletedByInference') alertModalDeletedByInference;
  @ViewChild('projectsModal') projectsModal;
  public deleteByInferenceOpened: boolean = false;
  
  //private ngUnsubscribe: Subject<void> = new Subject<void>();
  
  constructor(private element: ElementRef,
              private _organizationService: OrganizationService,
              private _piwikService: PiwikService,
              private  route: ActivatedRoute,
              private _searchDataprovidersService: SearchDataprovidersService,
              private _reportsService: ReportsService,
              private _searchResearchResultsService: SearchResearchResultsService,
              // private _searchDatasetsService: SearchDatasetsService,
              private _searchProjectsService: SearchProjectsService,
              private _meta: Meta,
              private _title: Title,
              private _router: Router,
              private helper: HelperService,
              private seoService: SEOService,
              private _location: Location) {
    this.fetchProjects = new FetchProjects(this._searchProjectsService);
  }
  
  ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties = data.envSpecific;
        if(this.properties.lastIndexUpdate) {
          this.indexUpdateDate = new Date(this.properties.lastIndexUpdate);
        }
        //this.getDivContents();
        this.getPageContents();
        this.updateUrl(data.envSpecific.baseLink + this._router.url);
      });
    this.sub = this.route.queryParams.subscribe(params => {
      this.organizationInfo = null;
      this.updateTitle("Organization");
      this.updateDescription("");
      
      this.organizationId = params['organizationId'];

      if (this.organizationId) {
        this.getOrganizationInfo();
      } else {
        this.showLoading = false;
        this._router.navigate(['/error'], {
          queryParams: {
            "page": this._location.path(true),
            "page_type": "organization"
          }
        });
        //this.warningMessage = "No valid organization id";
      }
      
      HelperFunctions.scroll();
      
      this.csvParamsTail = '" and relorganizationid exact "' + this.organizationId + '" ))';
      
    });
    
    this.downloadURLAPI = this.properties.csvAPIURL;
    //this.csvAffiliatedPublications = this.downloadURLAPI + "?format=csv&type=publications&fq=(((oaftype exact result) and (resulttypeid exact publication)) and (relorganizationid exact \"" + this.organizationId + "\"))";
    this.csvProjectParamsHead = 'format=csv&type=projects&fq=((funder exact "';
    //this.csvPublicationParamsHead = 'format=csv-special&type=publications&page=0&query=((((oaftype exact result) and (resulttypeid exact publication)) and (funderid exact ';
  }
  
  private getPageContents() {
    this.helper.getPageHelpContents(this.properties, this.communityId, this._router.url).subscribe(contents => {
      this.pageContents = contents;
    })
  }
  
  private getDivContents() {
    this.helper.getDivHelpContents(this.properties, this.communityId, this._router.url).subscribe(contents => {
      this.divContents = contents;
    })
  }
  
  
  ngOnDestroy() {
    if (this.sub) {
      this.sub.unsubscribe();
    }
    if (this.piwiksub) {
      this.piwiksub.unsubscribe();
    }
    if (this.infoSub) {
      this.infoSub.unsubscribe();
    }
    if (this.downloadFileSub) {
      this.downloadFileSub.unsubscribe();
    }
    if (this.downloadFilePiwikSub) {
      this.downloadFilePiwikSub.unsubscribe();
    }
    if (this.countProjectsSub) {
      this.countProjectsSub.unsubscribe();
    }
    if (this.countPublSub) {
      this.countPublSub.unsubscribe();
    }
    if (this.downloadProjectPublSub) {
      this.downloadProjectPublSub.unsubscribe();
    }
    
    /*
    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
    */
  }
  
  private getTotalResearchResults() {
    this._searchResearchResultsService.numOfEntityResults('publication',
      this.organizationInfo.objIdentifier, 'organization', this.properties).subscribe(total => {
      this.total.publications = total;
      this.total.results += total;
    });
    this._searchResearchResultsService.numOfEntityResults('dataset',
      this.organizationInfo.objIdentifier, 'organization', this.properties).subscribe(total => {
      this.total.datasets = total;
      this.total.results += total;
    });
    this._searchResearchResultsService.numOfEntityResults('software',
      this.organizationInfo.objIdentifier, 'organization', this.properties).subscribe(total => {
      this.total.software = total;
      this.total.results += total;
    });
    this._searchResearchResultsService.numOfEntityResults('other',
      this.organizationInfo.objIdentifier, 'organization', this.properties).subscribe(total => {
      this.total.other = total;
      this.total.results += total;
    });
  }
  
  private getTotalDataproviders() {
    this._searchDataprovidersService.numOfEntityDataproviders(
      this.organizationInfo.objIdentifier,'organization', this.properties).subscribe(
      total => {
        this.total.dataproviders = total;
      });
  }
  
  private getOrganizationInfo() {
    
    this.warningMessage = '';
    this.errorMessage = ""
    this.showLoading = true;
    
    this.organizationInfo = null;
    
    this.infoSub = this._organizationService.getOrganizationInfo(this.organizationId, this.properties).subscribe(
      data => {
        if (data == null) {
          this.showLoading = false;
          this._router.navigate(['/error'], {
            queryParams: {
              "page": this._location.path(true),
              "page_type": "organization"
            }
          });
          this.errorMessage = 'No organization found';
        } else {
          this.organizationInfo = data;
          this.seoService.createLinkForCanonicalURL(this.properties.baseLink + this.properties.searchLinkToOrganization + this.organizationInfo.objIdentifier);
          this.updateTitle(this.organizationInfo.title.name);
          this.updateDescription("Organization, country, " + this.organizationInfo.title.name + ((this.organizationInfo.title.name && this.organizationInfo.title.name != this.organizationInfo.name) ? (", " + this.organizationInfo.name) : ""));
          if (this.properties.enablePiwikTrack && (typeof document !== 'undefined')) {
            this.piwiksub = this._piwikService.trackView(this.properties, this.organizationInfo.title.name, this.piwikSiteId).subscribe();
          }
          console.log(this.organizationInfo);
          var refineFields: string [] = ["funder"];
          this.getTotalResearchResults();
          this.getTotalDataproviders();
          this.fetchProjects.getResultsForOrganizations(this.organizationId, "", 1, 0, refineFields, this.properties);
          this.showLoading = false;
        }
      },
      err => {
        //console.log(err)
        this.handleError("Error getting organization for id: " + this.organizationId, err);
        if (err.status == 404) {
          this._router.navigate(['/error'], {
            queryParams: {
              "page": this._location.path(true),
              "page_type": "organization"
            }
          });
        }
        this.seoService.createLinkForCanonicalURL(this.properties.baseLink + this.properties.searchLinkToOrganizations);
        this.errorMessage = 'No organization found';
        this.showLoading = false;
      }
    );
  }
  
  
  /*public searchDataproviders(page: number = 1) {
    this.fetchDataproviders.getResultsForEntity("organization", this.organizationId, page, 1, this.properties);
  }*/
  
  public downloadFile(url: string, filename: string) {
    this.openLoading();
    this.setMessageLoading("Downloading CSV file");
    
    this.downloadFileSub = this._reportsService.downloadCSVFile(url).subscribe(
      data => {
        this.closeLoading();
        
        var url = window.URL.createObjectURL(data);
        var a = window.document.createElement('a');
        window.document.body.appendChild(a);
        a.setAttribute('style', 'display: none');
        a.href = url;
        a.download = filename + ".csv";
        a.click();
        window.URL.revokeObjectURL(url);
        a.remove(); // remove the element
        
        //window.open(window.URL.createObjectURL(data));
        if (this.properties.enablePiwikTrack && (typeof document !== 'undefined')) {
          this.downloadFilePiwikSub = this._piwikService.trackDownload(this.properties, url, this.piwikSiteId).subscribe();
        }
      },
      err => {
        //console.log("Error downloading the file.");
        this.handleError("Error downloading file: " + filename + ".csv", err);
        
        this.closeLoading();
        this.confirmOpenCsvError();
      }/*,
           () => console.log('Completed file download.')*/
    );
  }
  
  private downloadFileByFunder() {
    
    this.openLoading();
    this.setMessageLoading("Downloading CSV file");
    
    let response: string[] = [];
    let totalResponse: string = "";
    let projects = [];
    let counter: number = this.funderCount;
    let title: boolean = false;
    let title_index: number = 0;
    
    let filename: string = 'funder-project-' + this.funderContentType + '-report';
    
    this.countProjectsSub = this._searchProjectsService.getProjectsForOrganizations(this.organizationId, ' and (funder exact "' + this.encodeURI(this.funderId) + '" ) ', 1, this.funderCount, [], this.properties).subscribe(
      data => {
        projects = data[1];
        for (let index = 0; index < projects.length; index++) {
          this.countPublSub = this._searchResearchResultsService.numOfEntityResults(this.funderContentType, projects[index].id, "project", this.properties).subscribe(
            data => {
              //if(data == 0 && title) {   // if no publications for this project
              if (data == 0 && (counter > 1 || title)) {   // if no publications for this project
                counter--;
                response[index] = "";
                //console.info("index: "+index, "counter: "+counter, "id:"+projects[index].id, response[index]);
                
                if (counter == 0) {
                  //for(let i=count-1; i>=0; i--) {
                  for (let i = 0; i < projects.length; i++) {
                    if (response[i] != "") {
                      if (i == title_index) {
                        totalResponse = response[i] + totalResponse;
                      } else {
                        totalResponse += response[i];
                      }
                    }
                  }
                  this.closeLoading();
                  
                  var csvurl = window.URL.createObjectURL(new Blob([totalResponse], {type: 'text/csv'}));
                  var a = window.document.createElement('a');
                  window.document.body.appendChild(a);
                  a.setAttribute('style', 'display: none');
                  a.href = csvurl;
                  a.download = filename + ".csv";
                  a.click();
                  window.URL.revokeObjectURL(csvurl);
                  a.remove(); // remove the element
                }
              } else {
                let url: string;
                if (!title) {
                  title_index = index;
                  //console.info(title_index);
                  url = this.downloadURLAPI + '?format=csv-special&' + this.getTypeParam(this.funderContentType)+ '&fq=((relprojectid exact "' + projects[index].id + '"))';
                } else {
                  url = this.downloadURLAPI + '?format=csv-special-notitle&' + this.getTypeParam(this.funderContentType)+ '&fq=((relprojectid exact "' + projects[index].id + '"))';
                }
                title = true;
                
                this.downloadProjectPublSub = this._reportsService.getCSVResponse(url).subscribe(
                  data => {
                    counter--;
                    response[index] = data;
                    //console.info("index: "+index, "counter: "+counter, "id:"+projects[index].id, response[index]);
                    
                    if (counter == 0) {
                      //for(let i=count-1; i>=0; i--) {
                      for (let i = 0; i < projects.length; i++) {
                        if (response[i] != "") {
                          if (i == title_index) {
                            totalResponse = response[i] + totalResponse;
                          } else {
                            totalResponse += response[i];
                          }
                        }
                      }
                      this.closeLoading();
                      
                      var csvurl = window.URL.createObjectURL(new Blob([totalResponse], {type: 'text/csv'}));
                      var a = window.document.createElement('a');
                      window.document.body.appendChild(a);
                      a.setAttribute('style', 'display: none');
                      a.href = csvurl;
                      a.download = filename + ".csv";
                      a.click();
                      window.URL.revokeObjectURL(csvurl);
                      a.remove(); // remove the element
                    }
                  },
                  err => {
                    this.handleError("Error downloading file: " + filename, err);
                    
                    this.closeLoading();
                    this.confirmOpenCsvError();
                  }/*,
                            () => console.log('Completed file download.')*/
                );
              }
            },
            err => {
              this.handleError("Error getting number of publications for project with id: " + projects[index].id, err);
            });
        }
      },
      err => {
        this.handleError("Error getting projects for organization with id: " + this.organizationId, err);
        
        this.closeLoading();
        this.confirmOpenCsvError();
      }
    );
  }
  
  private updateDescription(description: string) {
    this._meta.updateTag({content: description.substring(0, 160)}, "name='description'");
    this._meta.updateTag({content: description.substring(0, 160)}, "property='og:description'");
  }
  
  private updateTitle(title: string) {
    var _prefix = "";
    // if(!this.communityId) {
    //   _prefix = "OpenAIRE | ";
    // }
    // var _title = _prefix + ((title.length > 50) ? title.substring(0, 50) : title);
    this._title.setTitle(title);
    this._meta.updateTag({content: title}, "property='og:title'");
  }
  
  private updateUrl(url: string) {
    this._meta.updateTag({content: url}, "property='og:url'");
  }
  
  private openLoading() {
    if (this.loading) {
      this.loading.open();
    }
  }
  
  private closeLoading() {
    if (this.loading) {
      this.loading.close();
    }
  }
  
  private setMessageLoading(message: string) {
    if (this.loading) {
      this.loading.message = message;
    }
  }
  
  public confirmOpenApplyAll(contentType: string) {
    this.alertApplyAll.cancelButton = true;
    this.alertApplyAll.okButton = true;
    this.alertApplyAll.alertTitle = "CSV FILE";
    this.alertApplyAll.message = "Do you wish to download a CSV file? Note that this process may take a while.";
    this.alertApplyAll.okButtonText = "Yes";
    this.alertApplyAll.cancelButtonText = "No";
    this.alertApplyAll.open();
    this.funderId = this.funder.id;
    this.funderCount = this.funder.number;
    this.funderContentType = contentType;
  }
  
  public confirmCloseApplyAll() {
    this.downloadFileByFunder();
  }
  
  public confirmOpenCsvError() {
    this.alertCsvError.cancelButton = false;
    this.alertCsvError.okButton = true;
    this.alertCsvError.alertTitle = "ERROR DOWNLOADING CSV FILE";
    this.alertCsvError.message = "There was an error in csv downloading. Please try again later.";
    this.alertCsvError.okButtonText = "OK";
    this.alertCsvError.open();
  }
  
  encodeURI(input: string): string {
    return StringUtils.URIEncode(input);
  }
  
  private handleError(message: string, error) {
    console.error("Organizaton Landing Page: " + message, error);
  }
  
  openDeletedByInference() {
    this.deleteByInferenceOpened = true;
    this.alertModalDeletedByInference.cancelButton = false;
    this.alertModalDeletedByInference.okButton = false;
    this.alertModalDeletedByInference.alertTitle = "Other versions of";
    this.alertModalDeletedByInference.open();
  }
  
  openProjectsModal() {
    this.projectsInModalComponent.pageChange({value: 1});
    this.projectsModal.cancelButton = false;
    this.projectsModal.okButton = false;
    this.projectsModal.alertTitle = "Projects of";
    this.projectsModal.open();
  }
  
  public getTypeParam(type: string): string {
    if(type == 'results') {
      type = 'publications&type=datasets&type=software&type=other';
    }
    return 'type='+type;
  }
  
  getCSVAffiliated(contentType: string): string {
    return this.downloadURLAPI + '?format=csv&' + this.getTypeParam(contentType) + '&fq=(relorganizationid exact "' + this.organizationId + '")';
  }
  
  getFunderProjects(): string {
    return this.downloadURLAPI + '?'+this.csvProjectParamsHead+encodeURI(this.funder.id)+this.csvParamsTail;
  }
  
  open(value: any) {
    console.log(value);
  }
}
